/* ********************************************************************************
 * Copyright 2017-2020 VMware, Inc.  All rights reserved. -- VMware Confidential
 * ********************************************************************************//**
 * html5mmrPlayer.js
 *
 *   Provides video playback on the client device.
 *//*
 * VideoSourceState
 *
 *   The state of the Video Source.
 */
 const VideoSourceState = {
    NORMAL: 0,
    STARTED: 1,
    STOPPED: 2
  },
  SourceBufferOperationType = {
    APPEND: 0,
    ABORT: 1
  },
  INVALID_VIDEOSOURCE_ID = -1;
var Logger = {
    error: function(a) {
      Logger.log(a, WebCommandId.HTMLMMR_WEB_COMMAND_FATAL_ERROR)
    },
    info: function(a) {
      Logger.log(a, WebCommandId.HTML5MMR_WEB_COMMAND_LOG_MESSAGE)
    },
    sendToConsole: function(a) {
      !console || console.log(a)
    },
    sendToWebsocket: function(a) {
      typeof a == "string" && (!gWebSocket || gWebSocket.readyState != WebSocket.OPEN ? Logger.sendToConsole("Websocket is not open. Cannot send message : " + a) : gWebSocket.send(a))
    },
    log: function(a, b) {
      try {
        if (typeof a == "string" || a.length > 0) {
          var c = {
            commandId: b,
            message: a
          };
          Logger.sendToConsole(JSON.stringify(c)), Logger.sendToWebsocket(JSON.stringify(c))
        }
      } catch (d) {
        Logger.sendToConsole("Exception occured : " + d.message)
      }
    }
  },
  Html5MMRPlayer = function() {
    var a = this;
    this._videoSources = {}, this._currentVideoSource = null, this._currentVideoSourceId = INVALID_VIDEOSOURCE_ID, this._currentSeqNum = 0, this._isPlaying = !1, this._isPlayPending = !1;
    var b = function() {
        var a = document.querySelector("video");
        if (!a) {
          a = document.createElement("video"), a.style.position = "absolute", a.style.top = "0px", a.style.left = "0px", gVideo = a, document.body.insertBefore(a, document.body.firstChild);
          var b = function() {
            gVideo && (gVideo.width = window.innerWidth, gVideo.height = window.innerHeight)
          };
          window.addEventListener("resize", b, !1), b(), Logger.info("video element initialized")
        }
      },
      c = function() {
        var a = document.getElementById("spinner");
        if (!a) {
          a = document.createElement("div"), a.style.width = "100%", a.style.height = "100%", a.style.display = "inline-block", a.style.position = "fixed", a.style.visibility = "hidden", a.id = "spinner";
          var b = new Image;
          b.src = "spinner.gif", b.style.position = "absolute", b.style.top = "0px", b.style.bottom = "0px", b.style.left = "0px", b.style.right = "0px", b.style.margin = "auto", b.style.height = "auto", b.style.width = "auto", a.appendChild(b), document.body.insertBefore(a, document.body.firstChild)
        }
      };
    this.initialize = function() {
      return b(), c(), !gWebSocket || !gVideo || !gVideoInfo ? (Logger.error("initialize(): Failed to find window variables."), !1) : (Logger.info("initialize(): Initialize the Html5MMRPlayer object."), gWebSocket.onmessage = d, !0)
    };
    var d = function(a) {
        var b = a.data;
        typeof b == "string" ? f(b) : k(b)
      },
      e = function(a) {
        var b = 0;
        for (b = 0; b < gVideo.textTracks.length; b++)
          if (gVideo.textTracks[b].label === a) return gVideo.textTracks[b];
        return null
      },
      f = function(b) {
        if (!b || typeof b != "string" || b.length == 0) return Logger.error("processJSONMessage(): Invalid parameter."), !1;
        try {
          var c = null;
          c = JSON.parse(b);
          if (!c || c.command === undefined) return Logger.error("processJSONMessage(): Ill-formatted JSON data: " + b), !1;
          switch (c.command) {
            case "startUrl":
              if (c.videoSourceId === undefined) return Logger.error("processJSONMessage(): No videoSourceId present: " + b), !1;
              var d = g(c.videoSourceId);
              if (!(d instanceof VideoSource)) return Logger.error("processJSONMessage(): Could not find or create VideoSource: " + b), !1;
              a._currentVideoSource = d, a._currentVideoSourceId = c.videoSourceId;
              if (!a._currentVideoSource.start(c)) return !1;
              return a._isPlayPending && (Logger.info("processJSONMessage(): Found pending play in starturl. Playing video again"), a._isPlayPending = !1, i()), !0;
            case "stopUrl":
              if (!h(c)) return Logger.error("processJSONMessage(): Failed to process stopUrl, jsonMsg =" + c + ", currentSeqNum=" + a._currentSeqNum), !1;
              return !0;
            case "addSourceBuffer":
              if (!(a._currentVideoSource instanceof VideoSource)) return Logger.error("processJSONMessage(): There is no current video source: " + b), !1;
              return a._currentVideoSource.processAddSourceBufferMessage(c);
            case "abortSourceBuffer":
              if (!(a._currentVideoSource instanceof VideoSource)) return Logger.error("processJSONMessage(): There is no current video source: " + b), !1;
              return a._currentVideoSource.processAbortSourceBuffer(c);
            case "control":
              return j(c);
            case "addTextTrack":
              if (typeof c.kind != "string" || typeof c.label != "string" || typeof c.label != "string") return Logger.info("processJSONMessage(): WARNING, Invalid format: " + b), !1;
              var f = gVideo.addTextTrack(c.kind, c.label, c.language);
              if (!(f instanceof TextTrack)) return Logger.info("processJSONMessage(): WARNING, Failed to add text track: " + b), !1;
              return Logger.info("processJSONMessage(): Successfully added text track:" + c.label), !0;
            case "addCues":
              if (typeof c.label != "string") return Logger.info("processJSONMessage(): WARNING, Invalid label format: " + b), !1;
              if (!c.cues || !c.cues.length) return Logger.info("processJSONMessage(): WARNING, Invalid cues format: " + b), !1;
              var f = e(c.label);
              if (!(f instanceof TextTrack)) return Logger.info("processJSONMessage(): WARNING, Could not find TextTrack: " + b), !1;
              for (var k = 0; k < c.cues.length; k++) {
                var l = c.cues[k];
                if (l) {
                  if (typeof l.startTime != "number" || typeof l.endTime != "number" || typeof l.text != "string") {
                    Logger.info("processJSONMessage(): WARNING, Cannot process cue, invalid format.");
                    continue
                  }
                  var m = new VTTCue(l.startTime, l.endTime, l.text);
                  f.addCue(m)
                }
              }
              return !0;
            case "textTrackChange":
              if (typeof c.label != "string" || typeof c.mode != "string") return Logger.info("processJSONMessage(): WARNING, Invalid message format: " + b), !1;
              var f = e(c.label);
              if (!(f instanceof TextTrack)) return Logger.info("processJSONMessage(): WARNING, Could not find TextTrack: " + b), !1;
              return f.mode = c.mode, !0;
            default:
              return Logger.info("processJSONMessage(): WARNING, unknown command " + c), !1
          }
        } catch (n) {
          return Logger.error("processJSONMessage(): Exception " + n.message), !1
        }
      },
      g = function(b, c) {
        if (typeof b != "number") return Logger.error("getVideoSourceInstance(): Video source Id is not valid format."), null;
        var d = c === undefined ? !0 : c;
        return !a._videoSources.hasOwnProperty(b) && d && (a._videoSources[b] = new VideoSource(b)), a._videoSources[b]
      },
      h = function(b) {
        var c = parseInt(b.seqNum),
          d = b.reference === "true";
        return c <= a._currentSeqNum ? (Logger.info("processStopUrlMessage(): ignored [seqNum] = " + c + "reference = " + d), !0) : b.videoSourceId != a._currentVideoSourceId || !a._currentVideoSource instanceof VideoSource ? (Logger.info("processStopUrlMessage(): VideoSourceId = " + b.videoSourceId + " currentVideoSourceId = " + a._currentVideoSourceId), d ? !0 : !1) : (a._currentVideoSource.stop(), a._currentVideoSource = null, a._currentVideoSourceId = INVALID_VIDEOSOURCE_ID, a._currentSeqNum = c, !0)
      },
      i = function() {
        gVideo.play().then(function() {
          Logger.info("playVideo(): video is now playing"), a._isPlaying = !0
        }).catch(function(a) {
          Logger.info("playVideo(): error while trying to play: " + a.toString())
        })
      },
      j = function(b) {
        var c = b.seqNum === undefined ? 0 : parseInt(b.seqNum),
          d = b.reference === "true";
        if (c <= a._currentSeqNum) return Logger.info("processControlMessage(): ignored [seqNum] = " + c + "reference = " + d), !0;
        if (b === undefined || b.control === undefined) return !1;
        Logger.info("processControlMessage(): Control message: " + JSON.stringify(b));
        switch (b.control) {
          case "play":
            if (!a._currentVideoSource) {
              Logger.info("processControlMessage(): Video source is null, setting isPlayPending to true"), a._isPlayPending = !0;
              break
            }
            if (!!a._currentVideoSource && a._currentVideoSource._state != VideoSourceState.STARTED) {
              Logger.info("processControlMessage(): Video source is not in started state, setting isPlayPending to true"), a._isPlayPending = !0;
              break
            }
            i();
            break;
          case "pause":
            a._isPlayPending && (Logger.info("processControlMessage(): Resetting isPlayPending as pause message was received"), a._isPlayPending = !1), a._isPlaying && (gVideo.pause(), a._isPlaying = !1), Logger.info("[VERIFY] " + gVideoInfo.title + " " + gVideoInfo.url + " " + gVideo.currentTime);
            break;
          case "seeked":
            if (b.param === undefined || b.param.currentTime === undefined || typeof b.param.currentTime != "number") return !1;
            gVideo.currentTime = b.param.currentTime;
            break;
          case "volumechange":
            if (b.param === undefined || b.param.volume === undefined || typeof b.param.volume != "number") return !1;
            gVideo.volume = b.param.volume;
            break;
          case "muted":
            if (b.param === undefined || b.param.state === undefined || typeof b.param.state != "boolean") return !1;
            gVideo.muted = b.param.state;
            break;
          case "ratechange":
            if (b.param === undefined || b.param.playbackRate === undefined || typeof b.param.playbackRate != "number") return !1;
            gVideo.playbackRate = b.param.playbackRate;
            break;
          default:
            Logger.error("processControlMessage(): ERROR, Command not supported: " + b.control)
        }
        return a._currentSeqNum = c, !0
      },
      k = function(a) {
        var b = new Uint8Array(a, 0, 16);
        if (!b || b.length != 16) return Logger.error("processBlobMessage(): Invalid header."), !1;
        var c = g(b[0]);
        return c instanceof VideoSource ? c._state === VideoSourceState.STOPPED ? (Logger.info("Discard blob, video source stopped, Id = " + b[0]), !1) : b[2] === 1 ? c.processAbortSourceBuffer({
          id: b[1]
        }) : c.processBlobMessage(b[1], new Uint8Array(a, 16)) : (Logger.error("processBlobMessage():  Invalid video source instance."), !1)
      }
  },
  PlayerSourceBuffer = function(a) {
    this.rawBuffer = a, this.pendingBuffers = []
  },
  VideoSource = function(a) {
    this._mediaSource = null, this._playerSBList = {}, this._state = VideoSourceState.NORMAL, this._id = a;
    var b = function(a) {
        switch (a.target.error.code) {
          case a.target.error.MEDIA_ERR_ABORTED:
            Logger.error("The video playback was aborted.");
            break;
          case a.target.error.MEDIA_ERR_NETWORK:
            Logger.error("A network error caused the video download to fail part-way.");
            break;
          case a.target.error.MEDIA_ERR_DECODE:
            Logger.error("The video playback was aborted due to a corruption problem or because the video used features which browser did not support.");
            break;
          case a.target.error.MEDIA_ERR_SRC_NOT_SUPPORTED:
            Logger.error("The video could not be loaded, either because the server or network failed or because the format is not supported.");
            break;
          default:
            Logger.error("An unknown error occurred.")
        }
      },
      c = function() {
        var a = document.getElementById("spinner");
        a && (a.style.visibility = "hidden")
      };
    this.start = function(a) {
      if (!a || a.constructor !== Object) return Logger.error("start(): Invalid object."), !1;
      if (typeof a.id != "string" || a.id.length == 0) return Logger.error("start(): Incorrect Id param" + JSON.stringify(a)), !1;
      gVideo.onerror || (Logger.info("start(): Set the error handler of the video."), gVideo.onerror = b), gVideo.oncanplay || (Logger.info("start(): Set the canplay handler of the video."), gVideo.oncanplay = c);
      if (a.id.substring(0, 5) == "blob:") {
        if (this._mediaSource != null) return Logger.info("start(): MediaSource object is not null."), !0;
        window.MediaSource && (this._mediaSource = new MediaSource, gVideo.src = URL.createObjectURL(this._mediaSource))
      } else gVideo.src = a.id;
      var d = document.getElementById("spinner");
      return d && (d.style.visibility = "visible"), Logger.info("start(): Video set to source = " + gVideo.src), this._state = VideoSourceState.STARTED, !0
    }, this.stop = function() {
      return Logger.info("stop(): Stopping video source = " + this._id), gVideo.onerror && (Logger.info("stop(): Reset the error handler of the video."), gVideo.onerror = null), gVideo.src.substring(0, 5) == "blob:" && URL.revokeObjectURL(gVideo.src), gVideo.src = "", this._playerSBList = {}, this._mediaSource = null, this._state = VideoSourceState.STOPPED, !0
    }, this.updatePlayerSourceBufferFromId = function(a, b, c) {
      var d = " hint: " + c + ", Id = " + a;
      return typeof a != "number" ? (Logger.error("updatePlayerSourceBufferFromId(): Invalid id parameter." + d), null) : this._playerSBList.hasOwnProperty(a) ? (b && (Logger.info("updatePlayerSourceBufferFromId(): Update PlayerSourceBuffer, " + d), this._playerSBList[a].rawBuffer = b), this._playerSBList[a]) : (Logger.info("updatePlayerSourceBufferFromId(): Create new PlayerSourceBuffer, " + d), this._playerSBList[a] = new PlayerSourceBuffer(b), this._playerSBList[a])
    }, this.performSourceBufferOperation = function(a, b) {
      try {
        var c = null,
          d = " hint: " + b + ", Id = " + a;
        if (typeof a != "number") return Logger.error("performSourceBufferOperation(): Invalid parameter. " + d), !1;
        if (!this._playerSBList.hasOwnProperty(a)) return Logger.info("performSourceBufferOperation(): No source buffer of particular " + d), !1;
        c = this._playerSBList[a];
        if (!!c && c.rawBuffer instanceof SourceBuffer) {
          if (c.pendingBuffers.length > 0 && !c.rawBuffer.updating) {
            var e = c.pendingBuffers.shift();
            switch (e.type) {
              case SourceBufferOperationType.APPEND:
                e.buffer instanceof Uint8Array && (c.rawBuffer.appendBuffer(e.buffer), Logger.info("performSourceBufferOperation(): Append buffer " + d));
                break;
              case SourceBufferOperationType.ABORT:
                c.rawBuffer.abort(), Logger.info("performSourceBufferOperation(): Abort Source Buffer " + d);
                break;
              default:
                Logger.error("performSourceBufferOperation(): Invalid operation type. " + d)
            }
          }
          return !0
        }
        return Logger.info("performSourceBufferOperation(): Raw source buffer not instantiated yet." + d), !1
      } catch (f) {
        return Logger.error("performSourceBufferOperation: Exception:" + f.message + d), !1
      }
    }, this.processAddSourceBufferMessage = function(a) {
      var b = this,
        c = null;
      if (!a || typeof a.id != "number" || typeof a.mimeType != "string") return Logger.error("processAddSourceBufferMessage(): Invalid parameter."), !1;
      if (this._mediaSource == null) return Logger.error("processAddSourceBufferMessage(): Media Source is NULL."), !1;
      if (!MediaSource.isTypeSupported(a.mimeType)) return Logger.error("processAddSourceBufferMessage(): MimeType is not supported. MimeType = " + a.mimeType), !1;
      var d = function(e) {
        try {
          c = b._mediaSource.addSourceBuffer(a.mimeType), b.updatePlayerSourceBufferFromId(a.id, c, "sourceOpen"), c.id = a.id, Logger.info("sourceOpen(): Source buffer added, Id = " + a.id + ", mimeType = " + a.mimeType), c.addEventListener("updatestart", function(c) {
            b.performSourceBufferOperation(this.id, "updatestart")
          }), c.addEventListener("update", function(c) {
            b.performSourceBufferOperation(this.id, "update")
          }), c.addEventListener("updateend", function(c) {
            b.performSourceBufferOperation(this.id, "updateend")
          }), b.performSourceBufferOperation(a.id, "sourceOpen"), b._mediaSource.removeEventListener("sourceopen", d, !1)
        } catch (e) {
          Logger.error("sourceOpen(): Exception thrown:" + e.message + "Id = " + a.id)
        }
      };
      return this._mediaSource.readyState == "open" ? d() : this._mediaSource.addEventListener("sourceopen", d, !1), !0
    }, this.processAbortSourceBuffer = function(a) {
      if (!a || typeof a.id != "number") return Logger.error("processAbortSourceBuffer(): Invalid paramter."), !1;
      var b = this.updatePlayerSourceBufferFromId(a.id, null, "processAbortSourceBuffer");
      return b instanceof PlayerSourceBuffer ? (b.pendingBuffers.push({
        type: SourceBufferOperationType.ABORT
      }), this.performSourceBufferOperation(a.id, "processAbortSourceBuffer"), !0) : (Logger.error("processAbortSourceBuffer(): Cannot instantiate PlayerSourceBuffer from Id = " + a.id), !1)
    }, this.processBlobMessage = function(a, b) {
      if (typeof a == "number" && b instanceof Uint8Array) {
        var c = this.updatePlayerSourceBufferFromId(a, null, "processBlobMessage");
        return c instanceof PlayerSourceBuffer ? (c.pendingBuffers.push({
          type: SourceBufferOperationType.APPEND,
          buffer: b
        }), this.performSourceBufferOperation(a, "processBlobMessage"), !0) : (Logger.error("processBlobMessage(): Cannot instantiate PlayerSourceBuffer from Id = " + a), !1)
      }
      return Logger.error("processBlobMessage(): Invalid parameters."), !1
    }
  },
  gPlayer = new Html5MMRPlayer;
gPlayer.initialize();